## Cell Line expectations

CL_expect <- function(gct.file,removelist,sample.info.file,cn.segtab,CN.file, keep.list = NULL){

  ## For local troubleshooting.
  #gct.file = './input/Achilles_Achilles_filtered.gct'
  #removelist = './input/removelist_v3.13a.txt'
  #sample.info.file = './input/Achilles_samplefile_20160505.tsv'
  #keep.list = './input/v3.14a_keeplist_clean.txt'
  
# Reading gct file
print('Reading gct file.')
O <- MSIG.Gct2Frame(gct.file)
O <- O$names 

O <- as.data.frame(O,stringsAsFactors = FALSE)

# Reading Remove List
print('Reading Remove list for removeSamples.')
RL <- read.delim(removelist, header=F, row.names=1,  sep="\t", blank.lines.skip=T, comment.char="", as.is=T, na.strings = "", check.names=FALSE) 

RL <- apply(as.data.frame(rownames(RL)),1,remove_trailing,pattern = ' ') 
RL <- as.data.frame(RL, stringsAsFactors = FALSE)
colnames(RL) <- 'lines'


# Reading keep list
if(!is.null(keep.list)){
print("Reading samples to keep file.")	
rl <- read.delim(keep.list, header=F, row.names=1,  sep="\t", blank.lines.skip=T, comment.char="", as.is=T, na.strings = "", check.names=FALSE)
KL <- row.names(rl)
KL <- as.data.frame(KL, stringsAsFactors = FALSE)
}

# Reading sample info file
print('Reading sample info file for collapseReps.')
ai <- read.table(sample.info.file, sep='\t', header=T, row.names=1, quote='', as.is=T, fill=T, check.names = FALSE)
ai <- cbind(ai, proper.name=rownames(ai))
aname <-  strsplit(rownames(ai),"_")
SF <- matrix(data=0,nrow=length(rownames(ai)),ncol=1)
for (i in 1:length(rownames(ai)))
{ SF[i]<- aname[[i]][[1]] }
SF <- as.data.frame(SF,stringsAsFactors = FALSE)
colnames(SF) <- 'names'

# Reading seg.tab file 
print('Reading seg.tab file for CN_Cutoff.')
cn.seg.raw <- read.table(cn.segtab,header = T, sep = "\t", quote = "\"",dec = ",", fill = T, comment.char = "",stringsAsFactors = F, check.names = FALSE)

Seg <- cn.seg.raw$CCLE_name
Seg <- as.data.frame(Seg,stringsAsFactors = FALSE)
print(dim(Seg))
# Reading copynumber variants file
print('Reading CCLE copy number variants file for PertCNFilter.')
CNV <- MSIG.Gct2Frame.rownames(filename = CN.file)
CNV <- CNV$names
CNV <- as.data.frame(CNV,stringsAsFactors = FALSE)

# Perform set operations to determine the expected cell lines after each checkpoint
# A '_" indicates '-' as in set difference (precicesly A - A\capB) while an 'n' indicates intersection ('\cap')
# While specific reps are involved
# Remove DNA lines
DNA_rem <- grep('DNA',O$O)
O <- O$O[-DNA_rem]
O <- as.data.frame(O,stringsAsFactors = FALSE)
colnames(O) <- 'lines'

colnames(RL) <- 'lines'


O_RL <- as.data.frame(setdiff(O$lines,RL$lines),stringsAsFactors = F)

#O_RL = as.data.frame(O_RL)
rem <- intersect(O$lines,RL$lines)
print('The following lines will be removed at removeSamples')
print(rem)

# Need to clean up names for intersection with the sample info file (collapseReps)



if(!is.null(keep.list)){
  colnames(O_RL) <- 'lines'
  colnames(KL) <- 'lines'
  
  print('The following lines will be removed at keepSamples')
  print(setdiff(O_RL$lines,KL$lines))
  
  O_RL <- intersect(O_RL$lines,KL$lines)
  O_RL <- as.data.frame(O_RL,stringsAsFactors = FALSE)
}

O_RL <- apply(O_RL,1,function(x){toupper(clean_names(x))}) 
O_RL <- as.data.frame(O_RL,stringsAsFactors = F)

colnames(SF) <- 'lines'
colnames(O_RL) <- 'lines'
O_RLnSF <- intersect(O_RL$lines,SF$lines)
O_RLnSF <- as.data.frame(O_RLnSF,stringsAsFactors = FALSE)

print('The following will be removed at collapseReps')
#rem = apply(O_RL,1,function(x){grep(x,SF)}) %>% unlist()# %>% as.vector()
colnames(O_RLnSF) <- 'lines'
print(setdiff(O_RL$lines,SF$lines))

write.table(unique(O_RLnSF$lines),'Expected_noCNFilter.txt',quote = FALSE, row.names = FALSE, col.names = FALSE)
print('Expected cell lines with no CN filtering in Expected_noCNFilter.txt')


## Now to predict cell lines for the CN filtering branch

# Will keep things in the CCLE copynumber seg.tab file in CN cutoff
# This is getting confusing so we will call O_RLnSF  = A
A <- O_RLnSF

# Need to split lineage from the Seg file
Seg <- apply(Seg,1,function(x){strsplit(x,'_')[[1]][1]})
Seg <- as.data.frame(Seg,stringsAsFactors = F)
colnames(A) <- 'lines'
colnames(Seg) <- 'lines'
AnSeg <- intersect(A$lines,Seg$lines)
AnSeg <- as.data.frame(AnSeg,stringsAsFactors = F)
print('The following are not in the seg.tab file for CN_Cutoff:')
print(setdiff(A$lines,Seg$lines))

# Need to split lineage from the CNV file
CNV <- apply(CNV,1,function(x){strsplit(x,'_')[[1]][1]})
CNV <- as.data.frame(CNV,stringsAsFactors = F)
colnames(AnSeg) <- 'lines'
colnames(CNV) <- 'lines'
AnSegnCNV <- intersect(AnSeg$lines,CNV$lines)
AnSegnCNV <- as.data.frame(AnSegnCNV,stringsAsFactors = F)

print('The following are not in the copy number variants file for PertCNFilter:')
print(setdiff(AnSeg$lines,CNV$lines))


print('Expected cell lines after CN filtering in Expected_CNfiltered.txt')
write.table(unique(AnSegnCNV),'Expected_CNfiltered.txt',quote =FALSE,row.names = F,col.names = F)


}























clean_names <- function(name){
  
  # very brute force. A more elegant cleaning would be preferred.
  
  name <- strsplit(name,"311CAS")[[1]][1]
  name <- strsplit(name,"311Cas")[[1]][1]
  name <- strsplit(name,"311cas")[[1]][1]
  name <- strsplit(name,"311-CAS")[[1]][1]
  name <- strsplit(name,"311-Cas")[[1]][1]
  name <- strsplit(name,"311-cas")[[1]][1]
  name <- strsplit(name,"Rep")[[1]][1]
  name <- strsplit(name,"REP")[[1]][1]
  name <- strsplit(name,"rep")[[1]][1]
  name <- strsplit(name,"AVANA")[[1]][1]
  name <- strsplit(name,"Avana")[[1]][1]
  name <- strsplit(name,"avana")[[1]][1] 
  name <- strsplit(name,"101Cas")[[1]][1]
  
  clean <- name
  clean <- gsub('_','',clean)
  clean <- gsub('-','',clean)
  clean <- gsub(' ','',clean)
  clean <- gsub('/','',clean)
  clean <- gsub(';','',clean)
  clean <- gsub(')','',clean,fixed = TRUE)
  clean <- gsub('(','',clean,fixed = TRUE)
  clean <- gsub('.','',clean,fixed = TRUE)
  
  
  return(clean) 
  
}

